/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.common.AbstractSInfoProcessor;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultFacet;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.flood_duration.FloodDurationCalculationResult.ValueGetter;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.LongitudinalSectionGenerator;
import org.dive4elements.river.model.Attribute.AttributeKey;
import org.dive4elements.river.themes.ThemeDocument;

/**
 * Processor to generate the facet and data series of infrastructure flood heights
 *
 * @author Matthias Schäfer
 *
 */
public final class FloodHeightProcessor extends AbstractSInfoProcessor {

    private static final String FACET_FLOOD_HEIGHT_LEFT = "sinfo_facet_flood_height.left";

    private static final String FACET_FLOOD_HEIGHT_RIGHT = "sinfo_facet_flood_height.right";

    private static final String FACET_FLOOD_HEIGHT_DESCRIPTION = "sinfo_facet_flood_height.description";

    private static final String FACET_MAIN_VALUE_HEIGHT = "mainvalue.w";

    private static final String FACET_MAIN_VALUE_HEIGHT_DESCRIPTION = "mainvalue.w.description";

    private static final String I18N_AXIS_LABEL = LongitudinalSectionGenerator.I18N_YAXIS_LABEL;

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_FLOOD_HEIGHT_LEFT);
        HANDLED_FACET_TYPES.add(FACET_FLOOD_HEIGHT_RIGHT);
        HANDLED_FACET_TYPES.add(FACET_MAIN_VALUE_HEIGHT);
    }

    public FloodHeightProcessor() {
        super(I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    public static Facet createFloodHeightFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex) {

        if (facetIndex == 0) {
            final String description = Resources.getMsg(context.getMeta(), FACET_FLOOD_HEIGHT_DESCRIPTION, FACET_FLOOD_HEIGHT_DESCRIPTION,
                    SInfoResultType.localizeRiverside(context, AttributeKey.LEFT));
            return new SInfoResultFacet(facetIndex, resultIndex, FACET_FLOOD_HEIGHT_LEFT, description, I18N_AXIS_LABEL, ComputeType.ADVANCE, id, hash);
        }
        else {
            final String description = Resources.getMsg(context.getMeta(), FACET_FLOOD_HEIGHT_DESCRIPTION, FACET_FLOOD_HEIGHT_DESCRIPTION,
                    SInfoResultType.localizeRiverside(context, AttributeKey.RIGHT));
            return new SInfoResultFacet(facetIndex, resultIndex, FACET_FLOOD_HEIGHT_RIGHT, description, I18N_AXIS_LABEL, ComputeType.ADVANCE, id, hash);
        }
    }

    public static Facet createMainValueHeightFacet(final CallContext context, final String hash, final String id, final FloodDurationCalculationResult result,
            final int facetIndex, final int resultIndex, final int dataIndex) {
        final String description = Resources.getMsg(context.getMeta(), FACET_MAIN_VALUE_HEIGHT_DESCRIPTION, FACET_MAIN_VALUE_HEIGHT_DESCRIPTION,
                result.getMainValueLabel(dataIndex));

        return new FloodDurationFacet(facetIndex, resultIndex, dataIndex, FACET_MAIN_VALUE_HEIGHT, description, ComputeType.ADVANCE, I18N_AXIS_LABEL, hash, id);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        final String facetName = bundle.getFacetName();

        if (FACET_FLOOD_HEIGHT_LEFT.contentEquals(facetName)) {
            return buildInfrastructureSeries(generator, bundle, theme, visible, AttributeKey.LEFT);
        }

        if (FACET_FLOOD_HEIGHT_RIGHT.contentEquals(facetName))
            return buildInfrastructureSeries(generator, bundle, theme, visible, AttributeKey.RIGHT);

        if (FACET_MAIN_VALUE_HEIGHT.contentEquals(facetName)) {

            final AbstractCalculationResult data = getResult(generator, bundle);

            final FloodDurationFacet facet = (FloodDurationFacet) bundle.getFacet();
            final int index = facet.getWaterlevelIndex();

            if (data instanceof FloodDurationCalculationResult) {
                final FloodDurationCalculationResult.ValueGetter valuegetter = new ValueGetter() {
                    @Override
                    public double getValue(final DurationWaterlevel waterlevel) {
                        return waterlevel.getWaterlevel();
                    }
                };
                final double[][] points = ((FloodDurationCalculationResult) data).getMainValueDurationPoints(generator, valuegetter, index);
                return buildSeriesForType(points, generator, bundle, theme, visible, null);
            }
        }

        final String error = String.format("Unknown facet name: %s", facetName);
        throw new UnsupportedOperationException(error);
    }

    private String buildInfrastructureSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible,
            final AttributeKey riverside) {

        final FloodDurationCalculationResult data = (FloodDurationCalculationResult) getResult(generator, bundle);

        final double[][] points = data.getInfrastructurePoints(SInfoResultType.infrastructureHeight, riverside);

        return buildSeriesForType(points, generator, bundle, theme, visible, null);
    }
}